/*=============================================================================
        File: plib.h
     Purpose: Defines values needed by every module of NURBS++
    Revision: $Id: plib.h,v 1.2 2002/05/22 17:06:47 philosophil Exp $
  Created by: Philippe Lavoie          (21 May, 2002)
 Modified by: 

 Copyright notice:
          Copyright (C) 2002 Philippe Lavoie
 
	  This library is free software; you can redistribute it and/or
	  modify it under the terms of the GNU Library General Public
	  License as published by the Free Software Foundation; either
	  version 2 of the License, or (at your option) any later version.
 
	  This library is distributed in the hope that it will be useful,
	  but WITHOUT ANY WARRANTY; without even the implied warranty of
	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	  Library General Public License for more details.
 
	  You should have received a copy of the GNU Library General Public
	  License along with this library; if not, write to the Free
	  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
=============================================================================*/
#ifndef PLIB_HEADER
#define PLIB_HEADER

#define GCC_VERSION (__GNUC__ * 10000 \
                     + __GNUC_MINOR__ * 100 \
                     + __GNUC_PATCHLEVEL__)

#include "plib_config.h"

#endif
