/* include/plib_config.h.  Generated by configure.  */
/* include/plib_config.h.in.  Generated from configure.in by autoheader.  */
// define if you want to use exception handling
#define USE_EXCEPTION 1

// define if you want verbose exception handling
/* #undef VERBOSE_EXCEPTION */

// define if you have the ISO C++ friend declaration style
#define HAVE_ISO_FRIEND_DECL 1

// define if you can handle the template of template class
#define HAVE_TEMPLATE_OF_TEMPLATE 1

// define if you can use -fno-implicit-template
#define NO_IMPLICIT_TEMPLATES 1

// define if you can use -frepo
/* #undef INCLUDE_TEMPLATE_SOURCE */

// define if you want Image Magick
/* #undef WITH_IMAGE_MAGICK */

// define if you want to do range validity checking
// and also other debugging functions
#define DEBUG_PLIB 1

// Define if you want the matrix to be in column order rather than row order.
// Column ordering is used by fortran and OpenGL.
/* #undef COLUMN_ORDER */

// the name of the package
#define PACKAGE "nurbs++"

// the version of the package
#define VERSION "3.0.11"

// if the C++ compiler supports namespaces
#define HAVE_NAMESPACE 1

// Specify if OpenGL is present on this system
/* #undef WITH_OPENGL */

// Specify if you are using Solaris with egcs
/* #undef USING_GNU_SOLARIS */

// Specify if you are using DEC Alpha with egcs
/* #undef USING_GNU_DECALPHA */

// Specify if you are using SGI with egcs
/* #undef USING_GNU_SGI */

// Specify if you are using Linux (assumed with egcs)
/* #undef USING_LINUX */

// Specify if you have CppUnit installed
#define HAS_CPPUNIT 1

// Specify if you have abs defined for the Complex type
#define HAS_COMPLEX_ABS 1

// Specify if you have conf defined for the Complex type
#define HAS_COMPLEX_CONJ 1

/* Define to 1 if you have the <dlfcn.h> header file. */
#define HAVE_DLFCN_H 1

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the <memory.h> header file. */
#define HAVE_MEMORY_H 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Name of package */
#define PACKAGE "nurbs++"

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT ""

/* Define to the full name of this package. */
#define PACKAGE_NAME ""

/* Define to the full name and version of this package. */
#define PACKAGE_STRING ""

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME ""

/* Define to the version of this package. */
#define PACKAGE_VERSION ""

/* Define to 1 if you have the ANSI C header files. */
#define STDC_HEADERS 1

/* Version number of package */
#define VERSION "3.0.11"

/* Define to 1 if the X Window System is missing or not being used. */
/* #undef X_DISPLAY_MISSING */
